import * as React from 'react';
import * as enzyme from 'enzyme';
import Button from '..';
var prefixCls = 'dk-button';
describe('Button[Basic]', function () {
  test("should render ." + prefixCls + " in a Button", function () {
    var wrapper = enzyme.shallow(React.createElement(Button, null, "Default"));
    wrapper.find(Button).forEach(function (button) {
      expect(button.shallow().hasClass(prefixCls)).toBeTruthy();
    });
  });
});
describe('Button[Type]', function () {
  test('should make contact on prop and className', function () {
    var wrapper = enzyme.mount(React.createElement(React.Fragment, null, React.createElement(Button, null, "Default"), React.createElement(Button, {
      type: "primary"
    }, "Primary"), React.createElement(Button, {
      outline: true
    }, "Outline"), React.createElement(Button, {
      type: "primary",
      outline: true
    }, "Primary"), React.createElement(Button, {
      round: true
    }, "Round"), React.createElement(Button, {
      type: "primary",
      round: true
    }, "Primary")));
    wrapper.find("." + prefixCls).forEach(function (button) {
      var type = button.prop('type');
      var outline = button.prop('outline');
      var round = button.prop('round');
      expect(button.hasClass(prefixCls + "-" + type)).toBeTruthy();

      if (outline) {
        expect(button.hasClass(prefixCls + "-outline-" + type)).toBeTruthy();
      }

      if (round) {
        expect(button.hasClass(prefixCls + "-rounded")).toBeTruthy();
      }
    });
  });
});
describe('Button[Size]', function () {
  test('should set size class while has size prop', function () {
    var wrapper = enzyme.mount(React.createElement(React.Fragment, null, React.createElement(Button, {
      size: "large"
    }, "Large"), React.createElement(Button, {
      size: "default"
    }, "Default"), React.createElement(Button, {
      size: "small"
    }, "Small")));
    wrapper.find("." + prefixCls).forEach(function (button) {
      var size = button.prop('size');
      expect(button.hasClass(prefixCls + "-" + size)).toBeTruthy();
    });
  });
});
describe('Button[Click]', function () {
  test('should has click wave effect', function () {
    var mockCallBack = jest.fn();
    var button = enzyme.shallow(React.createElement(Button, {
      onClick: mockCallBack
    }, "Ok!"));
    button.find('button').simulate('click');
    expect(mockCallBack.mock.calls.length).toEqual(1);
  });
});