import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _assertThisInitialized from "@babel/runtime/helpers/esm/assertThisInitialized";
import _inheritsLoose from "@babel/runtime/helpers/esm/inheritsLoose";
import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import * as React from 'react';
import { createPortal } from 'react-dom';
import * as classNames from 'classnames';
import { CSSTransition } from 'react-transition-group';
import { isArray } from 'lodash-es';
import Icon from '../icon';
import Picker from './picker';
import Range from './range'; // import Input from '../input';

import { allPlaceholders, allFormats, isDate, addMonths, parseDate, formatDate, isSameMonth } from './utils';

var Calendar =
/*#__PURE__*/
function (_React$PureComponent) {
  _inheritsLoose(Calendar, _React$PureComponent);

  Calendar.getDerivedStateFromProps = function getDerivedStateFromProps(nextProps) {
    var state = {};

    if ('visible' in nextProps) {
      state.visible = !!nextProps.visible;
    }

    if ('value' in nextProps) {
      if (isArray(nextProps.value)) {
        if (isDate(nextProps.value[0]) && isDate(nextProps.value[1])) {
          state.value = nextProps.value;
          state.inputValue = nextProps.value;
        }
      } else if (isDate(nextProps.value)) {
        state.value = nextProps.value;
        state.inputValue = nextProps.value;
      } else if (nextProps.value === undefined) {
        state.inputValue = nextProps.range ? [] : '';
        state.value = nextProps.range ? [] : '';
      }
    }

    return state;
  };

  function Calendar(props) {
    var _this;

    _this = _React$PureComponent.call(this, props) || this;

    _defineProperty(_assertThisInitialized(_this), "calendarElement", void 0);

    _defineProperty(_assertThisInitialized(_this), "panelElement", void 0);

    _defineProperty(_assertThisInitialized(_this), "renderContent", function () {
      var _this$props = _this.props,
          range = _this$props.range,
          _this$props$type = _this$props.type,
          type = _this$props$type === void 0 ? 'day' : _this$props$type,
          attributes = _objectWithoutPropertiesLoose(_this$props, ["range", "type"]);

      var _this$state = _this.state,
          current = _this$state.current,
          value = _this$state.value;

      var format = _this.getFormat();

      var childProps = _extends({}, attributes, {
        type: type,
        format: format,
        current: current,
        value: value,
        onChange: _this.handleChange
      });

      return current instanceof Date ? React.createElement(Picker, _extends({}, childProps, {
        current: current
      })) : React.createElement(Range, _extends({}, childProps, {
        current: current
      }));
    });

    _defineProperty(_assertThisInitialized(_this), "getPosition", function () {
      var el = _this.calendarElement.current;
      var rect = el.getBoundingClientRect();
      var scrollTop = document.documentElement.scrollTop || document.body.scrollTop;
      var scrollLeft = document.documentElement.scrollLeft || document.body.scrollLeft;
      var left = scrollLeft + rect.left;
      var top = scrollTop + rect.top + rect.height;
      return {
        left: left,
        top: top
      };
    });

    _defineProperty(_assertThisInitialized(_this), "handleEnter", function () {
      var position = _this.getPosition();

      _this.setState({
        position: position
      });
    });

    _defineProperty(_assertThisInitialized(_this), "getPlaceholder", function () {
      var _this$props2 = _this.props,
          placeholder = _this$props2.placeholder,
          _this$props2$type = _this$props2.type,
          type = _this$props2$type === void 0 ? 'day' : _this$props2$type,
          range = _this$props2.range;

      if (placeholder !== undefined) {
        return placeholder;
      }

      if (range) {
        return allPlaceholders.range[type];
      }

      return allPlaceholders[type];
    });

    _defineProperty(_assertThisInitialized(_this), "getFormat", function () {
      var _this$props3 = _this.props,
          format = _this$props3.format,
          _this$props3$type = _this$props3.type,
          type = _this$props3$type === void 0 ? 'day' : _this$props3$type;

      if (format) {
        return format;
      }

      return allFormats[type];
    });

    _defineProperty(_assertThisInitialized(_this), "getCurrent", function () {
      var _this$props4 = _this.props,
          value = _this$props4.value,
          range = _this$props4.range,
          type = _this$props4.type;

      if (range) {
        var current = [new Date(), new Date()];

        if (value instanceof Array) {
          current = value.map(function (v) {
            return _this.getDateFromValue(v);
          });

          if (current.length === 1) {
            current[1] = new Date();
          }
        }

        if ((type === 'day' || type === 'datetime') && isSameMonth(current[0], current[1])) {
          current[1] = addMonths(current[0], 1);
        }

        if (type === 'datetime') {
          current[0].setHours(0);
          current[0].setMinutes(0);
          current[0].setSeconds(0);
          current[1].setHours(0);
          current[1].setMinutes(0);
          current[1].setSeconds(0);
        }

        return current;
      }

      return _this.getDateFromValue(value);
    });

    _defineProperty(_assertThisInitialized(_this), "handleInputFocus", function () {
      _this.setState({
        visible: true
      });
    });

    _defineProperty(_assertThisInitialized(_this), "handleInputChange", function (event, index) {
      var value = event.target.value;
      var inputValue = _this.state.inputValue;

      if (inputValue instanceof Array && index !== undefined) {
        inputValue[index] = value;
        var newInputValue = [].concat(inputValue);

        _this.setState({
          inputValue: newInputValue
        });
      } else {
        _this.setState({
          inputValue: value
        });
      }
    });

    _defineProperty(_assertThisInitialized(_this), "bindDocumentClick", function () {
      document.addEventListener('click', _this.handleDocumentClick);
    });

    _defineProperty(_assertThisInitialized(_this), "clearDocumentClick", function () {
      document.removeEventListener('click', _this.handleDocumentClick);
    });

    _defineProperty(_assertThisInitialized(_this), "handleDocumentClick", function (event) {
      var calendarEl = _this.calendarElement.current;
      var contentEl = _this.panelElement.current;
      var targetEl = event.target;

      if (!(targetEl === calendarEl || calendarEl && calendarEl.contains(targetEl) || targetEl === contentEl || contentEl && contentEl.contains(targetEl))) {
        _this.setState({
          visible: false
        });
      }
    });

    _defineProperty(_assertThisInitialized(_this), "handleChange", function (date, isSelect) {
      var _this$props5 = _this.props,
          onChange = _this$props5.onChange,
          type = _this$props5.type;

      var format = _this.getFormat();

      var current = date;

      if (date instanceof Array) {
        current = [].concat(date);

        if (isSameMonth(current[0], current[1])) {
          current[1] = addMonths(current[1], 1);
        }
      }

      if (type === 'time') {
        _this.setState({
          current: current,
          value: date,
          inputValue: isArray(date) ? date.map(function (v) {
            return formatDate(v, format);
          }) : formatDate(date, format),
          visible: !isSelect
        });
      } else if (isSelect) {
        _this.setState({
          current: current,
          value: date,
          inputValue: isArray(date) ? date.map(function (v) {
            return formatDate(v, format);
          }) : formatDate(date, format),
          visible: false
        });

        if (typeof onChange === 'function') {
          var _format = _this.getFormat();

          var _dateStr = isArray(date) ? date.map(function (d) {
            return formatDate(d, _format);
          }) : formatDate(date, _format);

          onChange(date, _dateStr);
        }
      } else {
        setTimeout(function () {
          _this.setState({
            current: current
          });
        }, 0);
      }
    });

    _this.calendarElement = React.createRef();
    _this.panelElement = React.createRef();
    _this.state = {
      current: _this.getCurrent(),
      visible: false,
      value: props.range ? [] : '',
      position: {
        top: 0,
        left: 0
      },
      inputValue: props.range ? [] : ''
    };
    return _this;
  }

  var _proto = Calendar.prototype;

  _proto.render = function render() {
    var _classNames, _classNames2;

    var _this$props6 = this.props,
        className = _this$props6.className,
        prefixCls = _this$props6.prefixCls,
        type = _this$props6.type,
        range = _this$props6.range,
        disabled = _this$props6.disabled,
        onChange = _this$props6.onChange,
        attributes = _objectWithoutPropertiesLoose(_this$props6, ["className", "prefixCls", "type", "range", "disabled", "onChange"]);

    var _this$state2 = this.state,
        position = _this$state2.position,
        inputValue = _this$state2.inputValue,
        visible = _this$state2.visible;
    var placeholder = this.getPlaceholder();
    var calendarClassName = classNames((_classNames = {}, _classNames["" + prefixCls] = true, _classNames[prefixCls + "-disabled"] = disabled, _classNames), className);
    var panelClassName = classNames((_classNames2 = {}, _classNames2[prefixCls + "-panel"] = true, _classNames2[prefixCls + "-panel-time"] = type === 'time', _classNames2[prefixCls + "-panel-range"] = !!range, _classNames2));
    var calendarNode = React.createElement(CSSTransition, {
      "in": visible,
      unmountOnExit: true,
      timeout: 300,
      classNames: prefixCls + "-panel",
      onEntered: this.bindDocumentClick,
      onExited: this.clearDocumentClick,
      onEnter: this.handleEnter
    }, React.createElement("div", {
      className: panelClassName,
      style: position,
      ref: this.panelElement
    }, this.renderContent()));
    var format = this.getFormat();
    return React.createElement("span", _extends({}, attributes, {
      className: calendarClassName,
      ref: this.calendarElement
    }), isArray(inputValue) ? React.createElement("div", {
      className: prefixCls + "-range"
    }, React.createElement("input", {
      placeholder: placeholder[0],
      onFocus: this.handleInputFocus,
      value: isDate(inputValue[0]) ? formatDate(inputValue[0], format) : inputValue[0],
      onChange: this.handleInputChange.bind(this, 0)
    }), React.createElement("span", null, "~"), React.createElement("input", {
      placeholder: placeholder[1],
      onFocus: this.handleInputFocus,
      value: isDate(inputValue[1]) ? formatDate(inputValue[1], format) : inputValue[1],
      onChange: this.handleInputChange.bind(this, 1)
    })) : React.createElement("input", {
      className: prefixCls + "-input",
      placeholder: placeholder,
      onFocus: this.handleInputFocus,
      value: isDate(inputValue) ? formatDate(inputValue, format) : inputValue,
      onChange: this.handleInputChange
    }), React.createElement(Icon, {
      type: type === 'time' ? 'clock' : 'calendar',
      className: prefixCls + "-icon"
    }), !disabled && createPortal(calendarNode, document.body));
  };

  _proto.getDateFromValue = function getDateFromValue(val) {
    if (val && typeof val === 'string') {
      return parseDate(val, this.getFormat());
    }

    return new Date();
  };

  return Calendar;
}(React.PureComponent);

_defineProperty(Calendar, "componentType", 'Calendar');

_defineProperty(Calendar, "defaultProps", {
  prefixCls: 'dk-calendar',
  type: 'day'
});

export default Calendar;