import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _assertThisInitialized from "@babel/runtime/helpers/esm/assertThisInitialized";
import _inheritsLoose from "@babel/runtime/helpers/esm/inheritsLoose";
import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import * as React from 'react';
import * as classNames from 'classnames';
import { isEqual } from 'lodash-es';
import { Provider as CheckboxProvider } from './context';

var CheckboxGroup =
/*#__PURE__*/
function (_React$PureComponent) {
  _inheritsLoose(CheckboxGroup, _React$PureComponent);

  CheckboxGroup.getDerivedStateFromProps = function getDerivedStateFromProps(nextProps, prevState) {
    var newValue = nextProps.value;

    if (!Array.isArray(newValue)) {
      newValue = [];
    }

    if (!isEqual(newValue, prevState.options)) {
      return {
        options: newValue
      };
    }

    return null;
  };

  function CheckboxGroup(props) {
    var _this;

    _this = _React$PureComponent.call(this, props) || this;

    _defineProperty(_assertThisInitialized(_this), "handleRawChange", function (checked, value) {
      var options = _this.state.options;
      var onChange = _this.props.onChange;
      var newOptions = !!checked ? Array.from(new Set([].concat(options, [value]))) : options.filter(function (option) {
        return option !== value;
      });

      _this.setState({
        options: newOptions
      });

      if (typeof onChange === 'function') {
        onChange(newOptions);
      }
    });

    _defineProperty(_assertThisInitialized(_this), "handleChecked", function (value) {
      var options = _this.state.options;
      return options.indexOf(value) !== -1;
    });

    var _options = props.defaultValue || props.value || [];

    _this.state = {
      options: _options
    };
    return _this;
  }

  var _proto = CheckboxGroup.prototype;

  _proto.render = function render() {
    var _classNames;

    var _this$props = this.props,
        className = _this$props.className,
        prefixCls = _this$props.prefixCls,
        children = _this$props.children,
        min = _this$props.min,
        max = _this$props.max,
        onChange = _this$props.onChange,
        value = _this$props.value,
        defaultValue = _this$props.defaultValue,
        attributes = _objectWithoutPropertiesLoose(_this$props, ["className", "prefixCls", "children", "min", "max", "onChange", "value", "defaultValue"]);

    var options = this.state.options;
    var groupClassName = classNames((_classNames = {}, _classNames[prefixCls + "-group"] = true, _classNames), className);
    return React.createElement("div", _extends({
      className: groupClassName
    }, attributes), React.createElement(CheckboxProvider, {
      value: {
        onRawChange: this.handleRawChange,
        checked: this.handleChecked,
        options: options,
        min: min,
        max: max
      }
    }, children));
  };

  return CheckboxGroup;
}(React.PureComponent);

_defineProperty(CheckboxGroup, "componentType", 'CheckboxGroup');

_defineProperty(CheckboxGroup, "defaultProps", {
  prefixCls: 'dk-checkbox',
  value: []
});

export default CheckboxGroup;