import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _extends from "@babel/runtime/helpers/esm/extends";
import _assertThisInitialized from "@babel/runtime/helpers/esm/assertThisInitialized";
import _inheritsLoose from "@babel/runtime/helpers/esm/inheritsLoose";
import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import * as React from 'react';
import * as classNames from 'classnames';
import { CSSTransition } from 'react-transition-group';
import { createConsumer } from './context';
import warning from '../utils/warning';
import { COMPONENT_TYPE, DEFAULT_TRIGGER } from './constants';

var FormItem =
/*#__PURE__*/
function (_React$Component) {
  _inheritsLoose(FormItem, _React$Component);

  function FormItem(props) {
    var _this;

    _this = _React$Component.call(this, props) || this;

    _defineProperty(_assertThisInitialized(_this), "getChildren", function () {
      var realRule = _this.getRule();

      var children = _this.props.children;
      var _this$state = _this.state,
          value = _this$state.value,
          status = _this$state.status;
      return React.Children.map(children, function (child) {
        var componentType = child.type.componentType;

        if (COMPONENT_TYPE[componentType]) {
          var newProps = _extends({}, child.props, {
            status: status,
            onChange: function onChange(value) {
              _this.handleChange(value);

              if (child.props.onChange) {
                child.props.onChange(value);
              }
            }
          });

          var trigger = Array.isArray(realRule.trigger) ? Array.from(new Set([].concat(DEFAULT_TRIGGER, realRule.trigger))) : DEFAULT_TRIGGER;

          if ([COMPONENT_TYPE.Input, COMPONENT_TYPE.Select, COMPONENT_TYPE.Calendar].indexOf(componentType) !== -1) {
            if (trigger.indexOf('blur') !== -1) {
              newProps.onBlur = function () {
                _this.handleBlurAndFocus();

                if (child.props.onBlur) {
                  var _child$props;

                  (_child$props = child.props).onBlur.apply(_child$props, arguments);
                }
              };
            }

            if (trigger.indexOf('focus') !== -1) {
              newProps.onFocus = function () {
                _this.handleBlurAndFocus();

                if (child.props.onFocus) {
                  var _child$props2;

                  (_child$props2 = child.props).onFocus.apply(_child$props2, arguments);
                }
              };
            }
          }

          if ([COMPONENT_TYPE.CheckboxGroup, COMPONENT_TYPE.RadioGroup, COMPONENT_TYPE.Input, COMPONENT_TYPE.Select, COMPONENT_TYPE.Calendar].indexOf(componentType) !== -1) {
            newProps.value = value;
          }

          if ([COMPONENT_TYPE.Switch, COMPONENT_TYPE.Checkbox, COMPONENT_TYPE.Radio].indexOf(componentType) !== -1) {
            newProps.checked = !!value;
          }

          return React.cloneElement(child, newProps);
        }

        return child;
      });
    });

    _defineProperty(_assertThisInitialized(_this), "getRequired", function () {
      var rule = _this.getRule();

      var required = _this.props.required;

      if (required !== undefined) {
        return required;
      }

      return rule.required;
    });

    _defineProperty(_assertThisInitialized(_this), "getRule", function () {
      var _this$props = _this.props,
          name = _this$props.name,
          rule = _this$props.rule,
          form = _this$props.form;

      if (rule) {
        return rule;
      }

      if (form && form.rules) {
        return form.rules[name] || {};
      }

      return {};
    });

    _defineProperty(_assertThisInitialized(_this), "handleChange", function (value) {
      var rule = _this.getRule();

      var required = _this.getRequired(); // for Checkbox & Radio


      if (value && value.target) {
        value = value.target.checked;
      }

      var isInvalid = Array.isArray(value) ? value.length === 0 : value === undefined || value === '';
      var newState = {
        isInvalid: isInvalid,
        value: value
      };

      if (required) {
        newState.message = rule.message;
        newState.status = isInvalid ? 'error' : 'default';
      } else {
        newState.status = 'default';
      }

      _this.setState(newState);

      _this.handleValidator(value);
    });

    _defineProperty(_assertThisInitialized(_this), "handleBlurAndFocus", function () {
      var rule = _this.getRule();

      var required = _this.getRequired();

      var value = _this.state.value;

      if (rule && rule.message && required && !value) {
        _this.setState({
          message: rule.message,
          status: 'error',
          isInvalid: true
        });
      } else {
        _this.setState({
          status: 'default',
          isInvalid: false
        });
      }

      _this.handleValidator(value);
    });

    _defineProperty(_assertThisInitialized(_this), "handleValidator", function (value) {
      var form = _this.props.form;

      var rule = _this.getRule();

      var required = _this.getRequired();

      if (typeof rule.validator === 'function') {
        var values = form.getFieldsValues();
        var validatorMessage;
        rule.validator(values, value, function (message) {
          if (message && required) {
            _this.setState({
              message: message,
              isInvalid: true,
              status: 'error'
            });

            validatorMessage = message;
          }
        });
        return validatorMessage;
      }
    });

    _defineProperty(_assertThisInitialized(_this), "checkInvalid", function () {
      var rule = _this.getRule();

      var required = _this.getRequired();

      var name = _this.props.name;
      var value = _this.state.value;
      var isInvalid = Array.isArray(value) ? value.length === 0 : value === undefined || value === '';

      if (required) {
        var result = [];

        if (isInvalid) {
          warning(false, 'Form.Item.validator', name + " is invalid.");

          _this.setState({
            message: rule.message,
            status: 'error',
            isInvalid: true
          });

          result.push(rule.message);
        }

        var message = _this.handleValidator(value);

        if (message) {
          result.push(message);
        }

        if (result.length > 0) {
          return {
            message: result,
            name: name
          };
        }
      }

      return null;
    });

    _defineProperty(_assertThisInitialized(_this), "resetField", function () {
      _this.setState({
        status: 'default',
        isInvalid: false,
        value: undefined
      });
    });

    _this.state = {
      status: 'default',
      value: undefined,
      message: props.rule && props.rule.message ? props.rule.message : '',
      isInvalid: false
    };
    return _this;
  }

  var _proto = FormItem.prototype;

  _proto.componentDidMount = function componentDidMount() {
    var _this$props2 = this.props,
        form = _this$props2.form,
        name = _this$props2.name;
    var rule = this.getRule();

    if (form && name) {
      var _form$addField;

      form.addField((_form$addField = {}, _form$addField[name] = {
        name: name,
        rule: rule,
        component: this
      }, _form$addField));
    }
  };

  _proto.componentWillUnmount = function componentWillUnmount() {
    var _this$props3 = this.props,
        form = _this$props3.form,
        name = _this$props3.name;

    if (form && name) {
      form.removeField(name);
    }
  };

  _proto.render = function render() {
    var _classNames, _classNames2;

    var _this$props4 = this.props,
        children = _this$props4.children,
        prefixCls = _this$props4.prefixCls,
        className = _this$props4.className,
        labelAlign = _this$props4.labelAlign,
        labelWidth = _this$props4.labelWidth,
        labelClassName = _this$props4.labelClassName,
        name = _this$props4.name,
        label = _this$props4.label,
        rule = _this$props4.rule,
        form = _this$props4.form,
        statusProp = _this$props4.status,
        attributes = _objectWithoutPropertiesLoose(_this$props4, ["children", "prefixCls", "className", "labelAlign", "labelWidth", "labelClassName", "name", "label", "rule", "form", "status"]);

    var _this$state2 = this.state,
        message = _this$state2.message,
        isInvalid = _this$state2.isInvalid;
    var required = this.getRequired();
    var itemClassName = classNames((_classNames = {}, _classNames[prefixCls + "-item"] = true, _classNames[prefixCls + "-item-top"] = labelAlign === 'top', _classNames[prefixCls + "-item-required"] = this.getRequired(), _classNames), className);
    var labelClsName = classNames((_classNames2 = {}, _classNames2[prefixCls + "-item-label"] = true, _classNames2[prefixCls + "-item-label-required"] = required, _classNames2[prefixCls + "-item-label-" + labelAlign] = true, _classNames2), labelClassName);
    var newChildren = this.getChildren();
    var messageNode = React.createElement(CSSTransition, {
      "in": !!(message && isInvalid),
      timeout: 216,
      unmountOnExit: true,
      classNames: prefixCls + "-item-message"
    }, React.createElement("div", {
      className: classNames(prefixCls + "-item-message", prefixCls + "-item-message-" + this.state.status)
    }, message));
    return React.createElement("div", _extends({
      className: itemClassName
    }, attributes), React.createElement("div", {
      className: labelClsName,
      style: {
        width: labelWidth
      }
    }, label), React.createElement("div", {
      className: prefixCls + "-item-control"
    }, newChildren, messageNode));
  };

  return FormItem;
}(React.Component);

_defineProperty(FormItem, "defaultProps", {
  prefixCls: 'dk-form',
  theme: 'default'
});

export default createConsumer(FormItem);