import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _assertThisInitialized from "@babel/runtime/helpers/esm/assertThisInitialized";
import _inheritsLoose from "@babel/runtime/helpers/esm/inheritsLoose";
import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import * as React from 'react';
import * as classNames from 'classnames';
import Icon from '../icon';

function fixControlledValue(value) {
  if (typeof value === 'undefined' || value === null) {
    return '';
  }

  return value;
}

var Input =
/*#__PURE__*/
function (_React$Component) {
  _inheritsLoose(Input, _React$Component);

  Input.getDerivedStateFromProps = function getDerivedStateFromProps(nextProps) {
    if ('value' in nextProps) {
      return {
        value: nextProps.value
      };
    }

    return null;
  };

  function Input(props) {
    var _this;

    _this = _React$Component.call(this, props) || this;

    _defineProperty(_assertThisInitialized(_this), "handleChange", function (event) {
      var onChange = _this.props.onChange;
      var value = event.target.value;

      if (_this.invalidNumber(value)) {
        return;
      }

      if (onChange) {
        onChange(value);
      }
    });

    _defineProperty(_assertThisInitialized(_this), "handleBlur", function (event) {
      var value = event.target.value;
      var onBlur = _this.props.onBlur;

      if (onBlur) {
        onBlur(event);
      }

      if (_this.invalidNumber(value)) {
        return;
      }
    });

    _defineProperty(_assertThisInitialized(_this), "handleFocus", function (event) {
      var onFocus = _this.props.onFocus;

      if (onFocus) {
        onFocus(event);
      }
    });

    _defineProperty(_assertThisInitialized(_this), "invalidNumber", function (value) {
      var _this$props = _this.props,
          digits = _this$props.digits,
          type = _this$props.type;

      if (type !== 'number') {
        return false;
      }

      var reg = '^-?\\d*';

      if (digits === undefined) {
        reg += '\\.?\\d*';
      } else if (digits > 0) {
        reg += "\\.?\\d{0," + digits + "}";
      }

      reg += '$';
      return !new RegExp(reg).test(value);
    });

    _this.state = {
      value: props.value
    };
    return _this;
  }

  var _proto = Input.prototype;

  _proto.render = function render() {
    var _classNames;

    var _this$props2 = this.props,
        children = _this$props2.children,
        className = _this$props2.className,
        disabled = _this$props2.disabled,
        size = _this$props2.size,
        type = _this$props2.type,
        prefixCls = _this$props2.prefixCls,
        prefix = _this$props2.prefix,
        suffix = _this$props2.suffix,
        prefixClassName = _this$props2.prefixClassName,
        suffixClassName = _this$props2.suffixClassName,
        _this$props2$wrapperP = _this$props2.wrapperProps,
        wrapperProps = _this$props2$wrapperP === void 0 ? {} : _this$props2$wrapperP,
        wrapperRef = _this$props2.wrapperRef,
        status = _this$props2.status,
        attributes = _objectWithoutPropertiesLoose(_this$props2, ["children", "className", "disabled", "size", "type", "prefixCls", "prefix", "suffix", "prefixClassName", "suffixClassName", "wrapperProps", "wrapperRef", "status"]);

    var value = this.props.value;
    var inputClassName = classNames(prefixCls, (_classNames = {}, _classNames[prefixCls + "-large"] = size === 'large', _classNames[prefixCls + "-small"] = size === 'small', _classNames[prefixCls + "-prefix"] = prefix, _classNames[prefixCls + "-suffix"] = suffix, _classNames[prefixCls + "-" + status] = status !== 'default', _classNames), className);
    var inputType = type === 'password' ? 'password' : 'text';

    if ('value' in this.props) {
      attributes.value = fixControlledValue(value);
      delete attributes.defaultValue;
    }

    var inputNode = React.createElement("input", _extends({}, attributes, {
      className: inputClassName,
      readOnly: disabled,
      type: inputType,
      onChange: this.handleChange,
      onBlur: this.handleBlur,
      onFocus: this.handleFocus
    }));

    var wrapperClassName = wrapperProps.className,
        otherWrapperProps = _objectWithoutPropertiesLoose(wrapperProps, ["className"]);

    return prefix || suffix ? React.createElement("div", _extends({
      className: classNames(prefixCls + "-wrapper", wrapperClassName)
    }, otherWrapperProps), prefix ? React.createElement(Icon, {
      type: prefix,
      className: classNames(prefixCls + "-prefix-icon", prefixClassName)
    }) : null, inputNode, suffix ? React.createElement(Icon, {
      type: suffix,
      className: classNames(prefixCls + "-suffix-icon", suffixClassName)
    }) : null) : inputNode;
  };

  return Input;
}(React.Component);

_defineProperty(Input, "componentType", 'Input');

_defineProperty(Input, "defaultProps", {
  prefixCls: 'dk-input',
  size: 'default',
  status: 'default',
  type: 'text'
});

export default Input;