import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _inheritsLoose from "@babel/runtime/helpers/esm/inheritsLoose";
import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import * as React from 'react';
import * as classNames from 'classnames';
import { MenuProvider } from './context';
import MenuItem from './item';
import ItemGroup from './item-group';
import SubMenu from './sub-menu';

var Menu =
/*#__PURE__*/
function (_React$Component) {
  _inheritsLoose(Menu, _React$Component);

  Menu.getDerivedStateFromProps = function getDerivedStateFromProps(nextProps) {
    if ('activeKey' in nextProps) {
      return {
        activeIndex: nextProps.activeKey
      };
    }

    return null;
  };

  function Menu(props) {
    var _this;

    _this = _React$Component.call(this, props) || this;
    _this.state = {
      activeIndex: props.activeKey || props.defaultActiveKey,
      openedMenus: props.defaultOpenKeys || []
    };
    return _this;
  }

  var _proto = Menu.prototype;

  _proto.render = function render() {
    var _classNames;

    var _this$props = this.props,
        children = _this$props.children,
        prefixCls = _this$props.prefixCls,
        className = _this$props.className,
        onOpen = _this$props.onOpen,
        onSelect = _this$props.onSelect,
        theme = _this$props.theme,
        mode = _this$props.mode,
        defaultActiveKey = _this$props.defaultActiveKey,
        defaultOpenKeys = _this$props.defaultOpenKeys,
        attributes = _objectWithoutPropertiesLoose(_this$props, ["children", "prefixCls", "className", "onOpen", "onSelect", "theme", "mode", "defaultActiveKey", "defaultOpenKeys"]);

    var menuClassName = classNames((_classNames = {}, _classNames["" + prefixCls] = true, _classNames[prefixCls + "-vertical"] = mode === 'vertical', _classNames[prefixCls + "-horizontal"] = mode === 'horizontal', _classNames[prefixCls + "-dark"] = theme === 'dark', _classNames), className);
    return React.createElement("ul", _extends({
      className: menuClassName
    }, attributes), React.createElement(MenuProvider, {
      value: this.getMenuContext()
    }, children));
  };

  _proto.getMenuContext = function getMenuContext() {
    var _this2 = this;

    return {
      getState: function getState() {
        return _this2.state;
      },
      getProps: function getProps() {
        return _this2.props;
      },
      selectItem: function selectItem(index) {
        var activeIndex = _this2.state.activeIndex;
        var onSelect = _this2.props.onSelect;

        if (index !== activeIndex) {
          _this2.setState({
            activeIndex: index
          });

          if (typeof onSelect === 'function') {
            onSelect(index);
          }
        }
      },
      addOpenedMenu: function addOpenedMenu(index) {
        var openedMenus = _this2.state.openedMenus;
        var onOpen = _this2.props.onOpen;

        _this2.setState({
          openedMenus: Array.from(new Set([].concat(openedMenus, [index])))
        });

        if (typeof onOpen === 'function') {
          onOpen(index);
        }
      },
      removeOpenedMenu: function removeOpenedMenu(index) {
        var openedMenus = _this2.state.openedMenus;

        _this2.setState({
          openedMenus: openedMenus.filter(function (m) {
            return m !== index;
          })
        });
      },
      existOpenedMenu: function existOpenedMenu(index) {
        return _this2.state.openedMenus.indexOf(index) !== -1;
      }
    };
  };

  return Menu;
}(React.Component);

_defineProperty(Menu, "Item", void 0);

_defineProperty(Menu, "ItemGroup", void 0);

_defineProperty(Menu, "SubMenu", void 0);

_defineProperty(Menu, "defaultProps", {
  prefixCls: 'dk-menu',
  mode: 'vertical',
  theme: 'light'
});

Menu.Item = MenuItem;
Menu.SubMenu = SubMenu;
Menu.ItemGroup = ItemGroup;
export default Menu;