import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _assertThisInitialized from "@babel/runtime/helpers/esm/assertThisInitialized";
import _inheritsLoose from "@babel/runtime/helpers/esm/inheritsLoose";
import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import * as React from 'react';
import * as classNames from 'classnames';
import { CSSTransition } from 'react-transition-group';
import { createPortal } from 'react-dom';
import { createConsumer } from './context';
import { removeClass } from '../utils/dom';
import Icon from '../icon';

var SubMenu =
/*#__PURE__*/
function (_React$Component) {
  _inheritsLoose(SubMenu, _React$Component);

  function SubMenu(props) {
    var _this;

    _this = _React$Component.call(this, props) || this;

    _defineProperty(_assertThisInitialized(_this), "titleRef", void 0);

    _defineProperty(_assertThisInitialized(_this), "hoverTimer", void 0);

    _defineProperty(_assertThisInitialized(_this), "position", void 0);

    _defineProperty(_assertThisInitialized(_this), "handleClick", function () {
      var _this$props = _this.props,
          index = _this$props.index,
          rootContext = _this$props.rootContext;

      if (!rootContext.existOpenedMenu(index)) {
        rootContext.addOpenedMenu(index);
      } else {
        rootContext.removeOpenedMenu(index);
      }
    });

    _defineProperty(_assertThisInitialized(_this), "handleMouseEnter", function () {
      var _this$props2 = _this.props,
          index = _this$props2.index,
          rootContext = _this$props2.rootContext;
      window.clearTimeout(_this.hoverTimer);
      _this.hoverTimer = window.setTimeout(function () {
        if (!rootContext.existOpenedMenu(index)) {
          _this.position = _this.getPosition();
          rootContext.addOpenedMenu(index);
          window.setTimeout(function () {
            _this.setState({
              active: true
            });
          }, 0);
        }
      }, 300);
    });

    _defineProperty(_assertThisInitialized(_this), "handleMouseLeave", function () {
      window.clearTimeout(_this.hoverTimer);
      _this.hoverTimer = window.setTimeout(function () {
        _this.setState({
          active: false
        });
      }, 300);
    });

    _defineProperty(_assertThisInitialized(_this), "handleEnter", function (el) {
      var prefixCls = _this.props.prefixCls;
      removeClass(el, prefixCls + "-submenu-list-opened");

      if (el.scrollHeight !== 0) {
        el.style.height = el.scrollHeight + 'px';
      }
    });

    _defineProperty(_assertThisInitialized(_this), "handleEntered", function (el) {
      el.style.height = '';
    });

    _defineProperty(_assertThisInitialized(_this), "handleExit", function (el) {
      el.style.height = el.scrollHeight + 'px';
    });

    _defineProperty(_assertThisInitialized(_this), "handleExiting", function (el) {
      if (el.scrollHeight !== 0) {
        el.style.height = '0';
      }
    });

    _defineProperty(_assertThisInitialized(_this), "handleExited", function () {
      var _this$props3 = _this.props,
          index = _this$props3.index,
          rootContext = _this$props3.rootContext;
      rootContext.removeOpenedMenu(index);
    });

    _defineProperty(_assertThisInitialized(_this), "getPosition", function () {
      var titleDiv = _this.titleRef.current;
      var left = 0;
      var top = 0;

      if (titleDiv && document.documentElement) {
        var rect = titleDiv.getBoundingClientRect();
        var scrollTop = document.documentElement.scrollTop || document.body.scrollTop;
        var scrollLeft = document.documentElement.scrollLeft || document.body.scrollLeft;
        left = scrollLeft + rect.left;
        top = scrollTop + rect.top + rect.height;
      }

      return {
        left: left,
        top: top
      };
    });

    _this.hoverTimer = 0;
    _this.titleRef = React.createRef();
    _this.position = {
      left: 0,
      top: 0
    };
    _this.state = {
      active: props.rootContext.existOpenedMenu(props.index)
    };
    return _this;
  }

  var _proto = SubMenu.prototype;

  _proto.render = function render() {
    var _classNames, _classNames2, _classNames3, _classNames4;

    var _this$props4 = this.props,
        children = _this$props4.children,
        index = _this$props4.index,
        prefixCls = _this$props4.prefixCls,
        className = _this$props4.className,
        icon = _this$props4.icon,
        title = _this$props4.title,
        rootContext = _this$props4.rootContext,
        attributes = _objectWithoutPropertiesLoose(_this$props4, ["children", "index", "prefixCls", "className", "icon", "title", "rootContext"]);

    var opened = rootContext.existOpenedMenu(index);
    var rootProps = rootContext.getProps();
    var isHorizontal = rootProps.mode === 'horizontal';
    var active = isHorizontal ? this.state.active : opened;
    var iconNode = icon && typeof icon === 'string' && React.createElement(Icon, {
      type: icon,
      className: prefixCls + "-icon"
    });
    var childNode = children && React.createElement(CSSTransition, {
      "in": active,
      timeout: 350,
      onEnter: this.handleEnter,
      onEntered: this.handleEntered,
      onExit: this.handleExit,
      onExiting: this.handleExiting,
      onExited: this.handleExited,
      classNames: prefixCls + "-submenu-list"
    }, React.createElement("ul", {
      className: classNames((_classNames = {}, _classNames[prefixCls + "-submenu-list"] = true, _classNames[prefixCls + "-dark-submenu-list"] = rootProps.theme === 'dark', _classNames[prefixCls + "-horizontal-submenu-list"] = isHorizontal, _classNames[prefixCls + "-submenu-list-opened"] = !isHorizontal && opened, _classNames)),
      style: isHorizontal ? this.position : undefined,
      onMouseEnter: isHorizontal ? this.handleMouseEnter : undefined,
      onMouseLeave: isHorizontal ? this.handleMouseLeave : undefined
    }, children));
    var titleClassName = classNames((_classNames2 = {}, _classNames2[prefixCls + "-submenu-title"] = true, _classNames2[prefixCls + "-horizontal-submenu-title"] = isHorizontal, _classNames2[prefixCls + "-active"] = opened, _classNames2));
    var titleNode = React.createElement("div", {
      className: titleClassName,
      onClick: !isHorizontal ? this.handleClick : null,
      onMouseEnter: isHorizontal ? this.handleMouseEnter : null,
      onMouseLeave: isHorizontal ? this.handleMouseLeave : null,
      ref: this.titleRef
    }, iconNode, title, React.createElement(Icon, {
      type: "chevron-down",
      className: classNames((_classNames3 = {}, _classNames3[prefixCls + "-arrow"] = true, _classNames3[prefixCls + "-arrow-active"] = active, _classNames3))
    }));
    return React.createElement("div", _extends({
      className: classNames((_classNames4 = {}, _classNames4[prefixCls + "-submenu"] = true, _classNames4), className)
    }, attributes), titleNode, !isHorizontal ? childNode : // : createPortal(childNode, document.body)
    opened ? createPortal(childNode, document.body) : null);
  };

  return SubMenu;
}(React.Component);

_defineProperty(SubMenu, "Item", void 0);

_defineProperty(SubMenu, "defaultProps", {
  prefixCls: 'dk-menu'
});

export default createConsumer(SubMenu);