import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _extends from "@babel/runtime/helpers/esm/extends";
import * as React from 'react';
import * as ReactDOM from 'react-dom';
import * as classNames from 'classnames';
import Panel from './panel';
import Button from '../button';
import Icon from '../icon';
export function create(type) {
  return function (options) {
    if (options === void 0) {
      options = {};
    }

    var div = document.createElement('div');
    document.body.appendChild(div);

    var props = _extends({
      prefixCls: 'dk-modal',
      width: 420
    }, options);

    ReactDOM.render(getPanel(props), div);
    setTimeout(function () {
      props.visible = true;
      ReactDOM.render(getPanel(props), div);
    }, 10);

    function destroy() {
      var unmountResult = ReactDOM.unmountComponentAtNode(div);

      if (unmountResult && div.parentNode) {
        div.parentNode.removeChild(div);
      }
    }

    function update(newProps) {
      if (newProps === void 0) {
        newProps = {};
      }

      props = _extends({}, props, newProps);
      ReactDOM.render(getPanel(props), div);
    }

    function getPanel(props) {
      var _classNames;

      var title = props.title,
          content = props.content,
          prefixCls = props.prefixCls,
          onConfirm = props.onConfirm,
          attributes = _objectWithoutPropertiesLoose(props, ["title", "content", "prefixCls", "onConfirm"]);

      var handleConfirmClick = function handleConfirmClick() {
        update({
          visible: false
        });

        if (typeof props.onConfirm === 'function') {
          props.onConfirm();
        }
      };

      var handleCancelClick = function handleCancelClick() {
        update({
          visible: false
        });

        if (typeof props.onCancel === 'function') {
          props.onCancel();
        }
      };

      var iconType = {
        success: 'check-circle',
        error: 'x-circle',
        warning: 'alert-circle',
        info: 'info',
        confirm: 'help-circle'
      };
      var contentNode = React.createElement("div", {
        className: prefixCls + "-body-container"
      }, React.createElement("span", {
        className: prefixCls + "-body-title"
      }, title), React.createElement("div", {
        className: prefixCls + "-body-content"
      }, content), React.createElement(Icon, {
        className: classNames(prefixCls + "-body-icon", (_classNames = {}, _classNames[prefixCls + "-body-icon-" + type] = true, _classNames)),
        type: iconType[type]
      }));
      return React.createElement(Panel, _extends({}, attributes, {
        showFooter: false,
        onClose: destroy
      }), contentNode, React.createElement("div", {
        className: prefixCls + "-body-btn"
      }, type === 'confirm' && React.createElement(Button, {
        onClick: handleCancelClick,
        className: prefixCls + "-body-btn-cancel"
      }, "Cancel"), React.createElement(Button, {
        type: "primary",
        onClick: handleConfirmClick
      }, "OK")));
    }

    return {
      destroy: destroy,
      update: update
    };
  };
}