import _assertThisInitialized from "@babel/runtime/helpers/esm/assertThisInitialized";
import _inheritsLoose from "@babel/runtime/helpers/esm/inheritsLoose";
import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import * as React from 'react';
import * as classNames from 'classnames';
import Icon from '../icon';

var Pagination =
/*#__PURE__*/
function (_React$Component) {
  _inheritsLoose(Pagination, _React$Component);

  Pagination.getDerivedStateFromProps = function getDerivedStateFromProps(nextProps) {
    var state = {};

    if ('current' in nextProps && nextProps.current) {
      state.current = nextProps.current;
    }

    return state;
  };

  function Pagination(props) {
    var _this;

    _this = _React$Component.call(this, props) || this;

    _defineProperty(_assertThisInitialized(_this), "getPages", function () {
      var _this$props = _this.props,
          total = _this$props.total,
          range = _this$props.range;
      var pageSize = _this.state.pageSize;
      var max = Math.ceil(total / pageSize);
      var pages = [];
      var current = _this.state.current;
      var left;
      var right;

      if (current > max) {
        current = max;
      }

      left = current - Math.floor(range / 2) + 1;

      if (left < 1) {
        left = 1;
      }

      right = left + range - 2;

      if (right >= max) {
        right = max;
        left = right - range + 2;

        if (left < 1) {
          left = 1;
        }
      } else {
        right -= left > 1 ? 1 : 0;
      }

      if (left > 1) {
        pages.push(1);
      }

      if (left > 2) {
        pages.push('<..');
      }

      for (var i = left; i < right + 1; i++) {
        pages.push(i);
      }

      if (right < max - 1) {
        pages.push('..>');
      }

      if (right < max) {
        pages.push(max);
      }

      return {
        pages: pages,
        max: max
      };
    });

    _this.state = {
      current: props.current || props.defaultCurrent,
      pageSize: props.pageSize
    };
    return _this;
  }

  var _proto = Pagination.prototype;

  _proto.render = function render() {
    var _classNames,
        _classNames2,
        _classNames3,
        _this2 = this,
        _classNames5;

    var _this$props2 = this.props,
        className = _this$props2.className,
        prefixCls = _this$props2.prefixCls,
        size = _this$props2.size,
        disabled = _this$props2.disabled;
    var current = this.state.current;
    var paginationClassName = classNames(prefixCls, (_classNames = {}, _classNames[prefixCls + "-large"] = size === 'large', _classNames[prefixCls + "-small"] = size === 'small', _classNames), className);
    var itemClassName = classNames(prefixCls + "-item", (_classNames2 = {}, _classNames2[prefixCls + "-disabled"] = !!disabled, _classNames2));

    var _this$getPages = this.getPages(),
        pages = _this$getPages.pages,
        max = _this$getPages.max;

    var items = [];
    items.push(React.createElement("button", {
      key: "previous",
      onClick: current > 1 && !disabled ? this.handleChange.bind(this, current - 1) : null,
      className: classNames(prefixCls + "-prev", (_classNames3 = {}, _classNames3[prefixCls + "-disabled"] = current === 1, _classNames3))
    }, React.createElement(Icon, {
      type: "chevron-left"
    })));
    pages.forEach(function (page) {
      if (page === '<..' || page === '..>') {
        items.push(React.createElement("div", {
          key: page,
          className: classNames(itemClassName, prefixCls + "-miss")
        }, React.createElement("i", null), React.createElement("i", null), React.createElement("i", null)));
      } else {
        var _classNames4;

        items.push(React.createElement("button", {
          key: page,
          onClick: current !== page && !disabled ? _this2.handleChange.bind(_this2, page) : null,
          className: classNames(itemClassName, (_classNames4 = {}, _classNames4[prefixCls + "-active"] = current === page, _classNames4))
        }, page));
      }
    });
    items.push(React.createElement("button", {
      key: "next",
      onClick: current < max && !disabled ? this.handleChange.bind(this, current + 1) : null,
      className: classNames(prefixCls + "-next", (_classNames5 = {}, _classNames5[prefixCls + "-disabled"] = current === max, _classNames5))
    }, React.createElement(Icon, {
      type: "chevron-right"
    })));
    return React.createElement("div", {
      className: paginationClassName
    }, items);
  };

  _proto.handleChange = function handleChange(current) {
    this.setState({
      current: current
    });

    if (this.props.onChange) {
      this.props.onChange(current);
    }
  };

  return Pagination;
}(React.Component);

_defineProperty(Pagination, "defaultProps", {
  prefixCls: 'dk-pagination',
  defaultCurrent: 1,
  pageSize: 20,
  total: 0,
  range: 7,
  size: 'default'
});

export default Pagination;