import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _assertThisInitialized from "@babel/runtime/helpers/esm/assertThisInitialized";
import _inheritsLoose from "@babel/runtime/helpers/esm/inheritsLoose";
import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import * as React from 'react';
import { createPortal, findDOMNode } from 'react-dom';
import { CSSTransition } from 'react-transition-group';
import * as classNames from 'classnames';
import { Provider } from './context';
import Input from '../input';

var Select =
/*#__PURE__*/
function (_React$PureComponent) {
  _inheritsLoose(Select, _React$PureComponent);

  Select.getDerivedStateFromProps = function getDerivedStateFromProps(nextProps) {
    if ('value' in nextProps) {
      if (nextProps.value === undefined) {
        return {
          inputValue: '',
          value: nextProps.value,
          options: nextProps.multiple ? [] : ''
        };
      }

      return {
        value: nextProps.value
      };
    }

    return null;
  };

  function Select(props) {
    var _this;

    _this = _React$PureComponent.call(this, props) || this;

    _defineProperty(_assertThisInitialized(_this), "selectElement", void 0);

    _defineProperty(_assertThisInitialized(_this), "panelElement", void 0);

    _defineProperty(_assertThisInitialized(_this), "bindDocumentClick", function () {
      document.addEventListener('click', _this.handleDocumentClick);
    });

    _defineProperty(_assertThisInitialized(_this), "clearDocumentClick", function () {
      document.removeEventListener('click', _this.handleDocumentClick);
    });

    _defineProperty(_assertThisInitialized(_this), "handleExited", function () {
      _this.clearDocumentClick();
    });

    _defineProperty(_assertThisInitialized(_this), "handleDocumentClick", function (event) {
      var el = findDOMNode(_this.selectElement.current);
      var contentEl = _this.panelElement.current;
      var targetEl = event.target;

      if (!(targetEl === el || el && el.contains(targetEl) || targetEl === contentEl || contentEl && contentEl.contains(targetEl))) {
        var options = _this.state.options;

        _this.setState({
          visible: false,
          inputValue: options instanceof Array ? '' : options.toString()
        });
      }
    });

    _defineProperty(_assertThisInitialized(_this), "getPosition", function () {
      var el = findDOMNode(_this.selectElement.current);
      var rect = el.getBoundingClientRect();
      var scrollTop = document.documentElement.scrollTop || document.body.scrollTop;
      var scrollLeft = document.documentElement.scrollLeft || document.body.scrollLeft;
      var left = scrollLeft + rect.left;
      var top = scrollTop + rect.top + rect.height;
      return {
        left: left,
        top: top
      };
    });

    _defineProperty(_assertThisInitialized(_this), "handleEnter", function () {
      var position = _this.getPosition();

      var el = findDOMNode(_this.selectElement.current);
      var width = el.clientWidth || 0;

      _this.setState({
        position: position,
        width: width
      });
    });

    _defineProperty(_assertThisInitialized(_this), "handleInputFocus", function () {
      _this.setState({
        visible: true,
        inputValue: ''
      });
    });

    _defineProperty(_assertThisInitialized(_this), "handleInputChange", function (value) {
      _this.setState({
        inputValue: value
      });
    });

    _defineProperty(_assertThisInitialized(_this), "handleRawChange", function (value) {
      var options = _this.state.options;
      var onChange = _this.props.onChange;
      var newOptions = options instanceof Array ? Array.from(new Set([].concat(options, [value]))) : value;
      var newInputValue = newOptions instanceof Array ? '' : value;

      _this.setState({
        options: newOptions,
        inputValue: newInputValue,
        visible: false
      });

      if (typeof onChange === 'function') {
        onChange(newOptions);
      }
    });

    _this.selectElement = React.createRef();
    _this.panelElement = React.createRef();
    _this.state = {
      visible: false,
      options: props.multiple ? [] : props.defaultValue || '',
      inputValue: props.defaultValue ? props.defaultValue.toString() : '',
      width: 0,
      position: {
        top: 0,
        left: 0
      }
    };
    return _this;
  }

  var _proto = Select.prototype;

  _proto.render = function render() {
    var _classNames;

    var _this$props = this.props,
        prefixCls = _this$props.prefixCls,
        size = _this$props.size,
        children = _this$props.children,
        disabled = _this$props.disabled,
        value = _this$props.value,
        multiple = _this$props.multiple,
        onChange = _this$props.onChange,
        prefix = _this$props.prefix,
        name = _this$props.name,
        prefixClassName = _this$props.prefixClassName,
        _this$props$placehold = _this$props.placeholder,
        placeholder = _this$props$placehold === void 0 ? 'Select' : _this$props$placehold,
        attributes = _objectWithoutPropertiesLoose(_this$props, ["prefixCls", "size", "children", "disabled", "value", "multiple", "onChange", "prefix", "name", "prefixClassName", "placeholder"]);

    var _this$state = this.state,
        position = _this$state.position,
        width = _this$state.width,
        options = _this$state.options,
        inputValue = _this$state.inputValue,
        visible = _this$state.visible;
    var iconClassName = classNames(prefixCls + "-icon", (_classNames = {}, _classNames[prefixCls + "-icon-visible"] = visible, _classNames));

    var panelStyle = _extends({}, position, {
      width: width
    });

    var selectNode = React.createElement(CSSTransition, {
      "in": visible,
      unmountOnExit: true,
      timeout: 300,
      classNames: prefixCls + "-panel",
      onEntered: this.bindDocumentClick,
      onExited: this.clearDocumentClick,
      onEnter: this.handleEnter
    }, React.createElement("div", {
      className: prefixCls + "-panel",
      style: panelStyle,
      ref: this.panelElement
    }, React.createElement(Provider, {
      value: {
        prefixCls: prefixCls,
        options: options,
        inputValue: inputValue,
        onRawChange: this.handleRawChange
      }
    }, children)));

    if (options instanceof Array) {
      return null;
    }

    return React.createElement(React.Fragment, null, React.createElement(Input, {
      placeholder: options.toString() || placeholder,
      value: inputValue,
      disabled: disabled,
      size: size,
      name: name,
      onChange: this.handleInputChange,
      onFocus: this.handleInputFocus,
      prefix: prefix,
      prefixClassName: prefixClassName,
      suffix: "chevron-down",
      wrapperProps: _extends({}, attributes, {
        ref: this.selectElement
      }),
      suffixClassName: iconClassName
    }), !disabled && createPortal(selectNode, document.body));
  };

  return Select;
}(React.PureComponent);

_defineProperty(Select, "Option", void 0);

_defineProperty(Select, "OptionGroup", void 0);

_defineProperty(Select, "componentType", 'Select');

_defineProperty(Select, "defaultProps", {
  prefixCls: 'dk-select',
  size: 'default'
});

export default Select;