import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _assertThisInitialized from "@babel/runtime/helpers/esm/assertThisInitialized";
import _inheritsLoose from "@babel/runtime/helpers/esm/inheritsLoose";
import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import * as React from 'react';
import * as classNames from 'classnames';
import { debounce } from 'lodash-es';
import Pagination from '../pagination';

var TableList =
/*#__PURE__*/
function (_React$Component) {
  _inheritsLoose(TableList, _React$Component);

  function TableList(props) {
    var _this;

    _this = _React$Component.call(this, props) || this;

    _defineProperty(_assertThisInitialized(_this), "lastScrollLeft", void 0);

    _defineProperty(_assertThisInitialized(_this), "lastScrollTop", void 0);

    _defineProperty(_assertThisInitialized(_this), "debouncedWindowResize", void 0);

    _defineProperty(_assertThisInitialized(_this), "resizeEvent", void 0);

    _defineProperty(_assertThisInitialized(_this), "bodyRef", void 0);

    _defineProperty(_assertThisInitialized(_this), "headRef", void 0);

    _defineProperty(_assertThisInitialized(_this), "handleBodyScrollLeft", function (e) {
      if (e.currentTarget !== e.target) {
        return;
      }

      var target = e.target;
      var _this$props$scroll = _this.props.scroll,
          scroll = _this$props$scroll === void 0 ? {} : _this$props$scroll;
      var headTable = _this.headRef.current;
      var bodyTable = _this.bodyRef.current;

      if (target.scrollLeft !== _this.lastScrollLeft && scroll.x) {
        if (target === bodyTable && headTable) {
          headTable.scrollLeft = target.scrollLeft;
        } else if (target === headTable && bodyTable) {
          bodyTable.scrollLeft = target.scrollLeft;
        }

        _this.setScrollPositionClassName();
      }

      _this.lastScrollLeft = target.scrollLeft;
    });

    _defineProperty(_assertThisInitialized(_this), "handleWindowResize", function () {
      _this.setScrollPositionClassName();
    });

    _this.lastScrollLeft = 0;
    _this.lastScrollTop = 0;
    _this.bodyRef = React.createRef();
    _this.headRef = React.createRef();
    _this.state = {
      position: 'both'
    };
    _this.debouncedWindowResize = debounce(_this.handleWindowResize, 150);
    return _this;
  }

  var _proto = TableList.prototype;

  _proto.componentDidMount = function componentDidMount() {
    this.handleWindowResize();
    this.resizeEvent = window.addEventListener('resize', this.debouncedWindowResize);
  };

  _proto.componentWillUnmount = function componentWillUnmount() {
    if (this.resizeEvent) {
      this.resizeEvent.remove();
    }

    if (this.debouncedWindowResize) {
      this.debouncedWindowResize.cancel();
    }
  };

  _proto.componentDidUpdate = function componentDidUpdate(prevProps) {
    if (this.props.fixed && prevProps.dataSource !== this.props.dataSource && this.bodyRef.current) {
      this.handleWindowResize();
      this.bodyRef.current.scrollTo(0, 0);
    }
  };

  _proto.render = function render() {
    var _classNames;

    var _this$props = this.props,
        prefixCls = _this$props.prefixCls,
        columns = _this$props.columns,
        title = _this$props.title,
        dataSource = _this$props.dataSource,
        className = _this$props.className,
        fixed = _this$props.fixed,
        _this$props$scroll2 = _this$props.scroll,
        scroll = _this$props$scroll2 === void 0 ? {} : _this$props$scroll2,
        pagination = _this$props.pagination,
        bodyClassName = _this$props.bodyClassName;
    var headCell = React.createElement("thead", null, React.createElement("tr", null, columns.map(function (column, index) {
      return React.createElement("th", {
        className: column.className,
        key: index
      }, column.title);
    })));
    var bodyCell = dataSource && dataSource.length > 0 && React.createElement("tbody", null, dataSource.map(function (itemData, index) {
      return React.createElement("tr", {
        key: index
      }, columns.map(function (column, key) {
        return React.createElement("td", {
          className: column.className,
          key: key
        }, column.render ? column.render(itemData, index, itemData[column.dataIndex]) : column.dataIndex && itemData[column.dataIndex] !== undefined ? String(itemData[column.dataIndex]) : null);
      }));
    }));

    var _ref = pagination || {},
        _ref$position = _ref.position,
        paginationPosition = _ref$position === void 0 ? 'bottom' : _ref$position,
        otherPaginationProps = _objectWithoutPropertiesLoose(_ref, ["position"]);

    var paginationCell = React.createElement("div", {
      className: prefixCls + "-table-page"
    }, React.createElement(Pagination, otherPaginationProps));
    var colgroupCell = React.createElement("colgroup", null, columns.map(function (c, index) {
      return React.createElement("col", {
        key: index,
        style: {
          width: c.width,
          minWidth: c.width
        }
      });
    }));
    var position = this.state.position;
    var tableClassName = classNames((_classNames = {}, _classNames["" + prefixCls] = true, _classNames[prefixCls + "-scroll"] = fixed, _classNames[prefixCls + "-position-left"] = position === 'left', _classNames[prefixCls + "-position-right"] = position === 'right', _classNames[prefixCls + "-position-middle"] = position === 'middle', _classNames[prefixCls + "-position-both"] = position === 'both', _classNames), className);
    var bodyStyle = {};
    var tableStyle = {};

    if (fixed && scroll.x) {
      bodyStyle.overflowX = 'scroll';
      bodyStyle.WebkitTransform = 'translate3d (0, 0, 0)';
      tableStyle.width = scroll.x;
      tableStyle.tableLayout = 'fixed';
    }

    if (fixed && scroll.y) {
      bodyStyle.overflowY = 'scroll';
      bodyStyle.maxHeight = scroll.y;
    }

    return React.createElement(React.Fragment, null, pagination && paginationPosition === 'top' ? paginationCell : null, React.createElement("div", {
      className: tableClassName
    }, title && React.createElement("div", {
      className: prefixCls + "-title"
    }, title), React.createElement("div", {
      className: prefixCls + "-header",
      ref: this.headRef,
      onScroll: this.handleBodyScrollLeft
    }, React.createElement("table", {
      style: tableStyle
    }, colgroupCell, headCell)), React.createElement("div", {
      className: classNames(prefixCls + "-body", bodyClassName),
      ref: this.bodyRef,
      style: bodyStyle,
      onScroll: this.handleBodyScrollLeft
    }, dataSource && dataSource.length ? React.createElement("table", {
      style: tableStyle
    }, colgroupCell, bodyCell) : React.createElement("div", {
      style: tableStyle,
      className: prefixCls + "-no-data"
    }, "No data")), fixed && React.createElement(React.Fragment, null, React.createElement("div", {
      className: prefixCls + "-fixed-left"
    }), React.createElement("div", {
      className: prefixCls + "-fixed-right"
    }))), pagination && paginationPosition === 'bottom' ? paginationCell : null);
  };

  _proto.setScrollPositionClassName = function setScrollPositionClassName() {
    var bodyTable = this.bodyRef.current;

    if (bodyTable) {
      var position = this.state.position;
      var scrollToLeft = bodyTable.scrollLeft === 0;
      var scrollToRight = bodyTable.scrollLeft + 1 >= bodyTable.children[0].getBoundingClientRect().width - bodyTable.getBoundingClientRect().width;

      if (scrollToLeft && scrollToRight) {
        this.setState({
          position: 'both'
        });
      } else if (scrollToLeft) {
        this.setState({
          position: 'left'
        });
      } else if (scrollToRight) {
        this.setState({
          position: 'right'
        });
      } else if (position !== 'middle') {
        this.setState({
          position: 'middle'
        });
      }
    }
  };

  return TableList;
}(React.Component);

_defineProperty(TableList, "defaultProps", {
  prefixCls: 'dk-table'
});

export default TableList;