"use strict";

exports.__esModule = true;
exports.shorten = shorten;
exports.getDaySuffix = getDaySuffix;
exports.pad = pad;
exports.toDate = toDate;
exports.addDays = addDays;
exports.addMonths = addMonths;
exports.addYears = addYears;
exports.endOfWeek = endOfWeek;
exports.endOfMonth = endOfMonth;
exports.startOfDay = startOfDay;
exports.startOfMonth = startOfMonth;
exports.startOfWeek = startOfWeek;
exports.isDate = isDate;
exports.isSameDay = isSameDay;
exports.isSameWeek = isSameWeek;
exports.isSameMonth = isSameMonth;
exports.isSameYear = isSameYear;
exports.compareAsc = compareAsc;
exports.getDayNumberOfMouth = getDayNumberOfMouth;
exports.getWeekNumberOfYear = getWeekNumberOfYear;
exports.prevMonth = prevMonth;
exports.nextMonth = nextMonth;
exports.prevYear = prevYear;
exports.nextYear = nextYear;
exports.getDaysOfMonth = getDaysOfMonth;
exports.formatDate = formatDate;
exports.parseDate = parseDate;
exports.allFormats = exports.allPlaceholders = exports.monthValues = exports.weekdayValues = void 0;
var token = /d{1,4}|M{1,4}|yy(?:yy)?|S{1,3}|Do|ZZ|WW|Wo|([HhMsDm])\1?|[aA]|"[^"]*"|'[^']*'/g;
var twoDigits = /\d\d?/;
var threeDigits = /\d{3}/;
var fourDigits = /\d{4}/; // tslint:disable-next-line

var word = /[0-9]*['a-z\u00A0-\u05FF\u0700-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]+|[\u0600-\u06FF\/]+(\s*?[\u0600-\u06FF]+){1,2}/i;
var literal = /\[([^]*?)\]/gm;
var amPm = ['am', 'pm'];
var weekdays = ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'];
var months = ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December'];
var weekdayValues = {
  "short": shorten(weekdays, 3),
  "long": weekdays
};
exports.weekdayValues = weekdayValues;
var monthValues = {
  "short": shorten(months, 3),
  "long": months
};
exports.monthValues = monthValues;
var allPlaceholders = {
  day: 'Select date',
  time: 'Select time',
  week: 'Select week',
  month: 'Select month',
  year: 'Select year',
  datetime: 'Select Datetime',
  range: {
    day: ['Start day', 'End day'],
    time: ['Start time', 'End time'],
    week: ['Start week', 'End week'],
    month: ['Start month', 'End month'],
    year: ['Start year', 'End year'],
    datetime: ['Start time', 'End time']
  }
};
exports.allPlaceholders = allPlaceholders;
var allFormats = {
  day: 'yyyy-MM-dd',
  time: 'HH:mm:ss',
  week: 'yyyy WW',
  month: 'yyyy-MM',
  year: 'yyyy',
  datetime: 'yyyy-MM-dd HH:mm:ss'
};
exports.allFormats = allFormats;

function shorten(arr, sLen) {
  var newArr = [];

  for (var i = 0, len = arr.length; i < len; i++) {
    newArr.push(arr[i].substr(0, sLen));
  }

  return newArr;
}

function getDaySuffix(d) {
  var s = ['th', 'st', 'nd', 'rd'];
  var v = d % 100;
  return d + (s[(v - 20) % 10] || s[v] || s[0]);
}

function pad(val, len) {
  if (len === void 0) {
    len = 2;
  }

  var newVal = String(val);

  while (newVal.length < len) {
    newVal = '0' + val;
  }

  return newVal;
}

function toDate(dirtyDate) {
  if (arguments.length < 1) {
    throw new TypeError('1 argument required, but only ' + arguments.length + ' present');
  }

  if (dirtyDate instanceof Date) {
    return new Date(dirtyDate.getTime());
  } else if (typeof dirtyDate === 'number' || Object.prototype.toString.call(dirtyDate) === '[object Number]') {
    return new Date(dirtyDate);
  } else if (typeof dirtyDate === 'string' || Object.prototype.toString.call(dirtyDate) === '[object String]') {
    return new Date(dirtyDate);
  }

  return new Date(NaN);
}

function addDays(dirtyDate, amount) {
  amount = Number(amount);
  var date = toDate(dirtyDate);
  date.setDate(date.getDate() + amount);
  return date;
}

function addMonths(dirtyDate, amount) {
  amount = Number(amount);
  var date = toDate(dirtyDate);
  var desiredMonth = date.getMonth() + amount;
  var dateWithDesiredMonth = new Date(0);
  dateWithDesiredMonth.setFullYear(date.getFullYear(), desiredMonth, 1);
  dateWithDesiredMonth.setHours(0, 0, 0, 0);
  var daysInMonth = getDayNumberOfMouth(dateWithDesiredMonth);
  date.setMonth(desiredMonth, Math.min(daysInMonth, date.getDate()));
  return date;
}

function addYears(dirtyDate, amount) {
  amount = Number(amount);
  return addMonths(dirtyDate, amount * 12);
}

function endOfWeek(dirtyDate) {
  var date = toDate(dirtyDate);
  var day = date.getDay();
  var diff = 6 - day;
  date.setDate(date.getDate() + diff);
  date.setHours(23, 59, 59, 999);
  return date;
}

function endOfMonth(dirtyDate) {
  var date = toDate(dirtyDate);
  var month = date.getMonth();
  date.setFullYear(date.getFullYear(), month + 1, 0);
  date.setHours(23, 59, 59, 999);
  return date;
}

function startOfDay(dirtyDate) {
  var date = toDate(dirtyDate);
  date.setHours(0, 0, 0, 0);
  return date;
}

function startOfMonth(dirtyDate) {
  var date = toDate(dirtyDate);
  date.setDate(1);
  date.setHours(0, 0, 0, 0);
  return date;
}

function startOfWeek(dirtyDate) {
  var date = toDate(dirtyDate);
  var day = date.getDay();
  var diff = day;
  date.setDate(date.getDate() - diff);
  date.setHours(0, 0, 0, 0);
  return date;
}

function isDate(dirtyDate) {
  if (dirtyDate === null || dirtyDate === undefined) {
    return false;
  }

  if (isNaN(new Date(dirtyDate).getTime())) {
    return false;
  }

  if (Array.isArray(dirtyDate)) {
    // deal with `new Date([ new Date() ]) -> new Date()`
    return false;
  }

  return true;
}

function isSameDay(dirtyDateLeft, dirtyDateRight) {
  if (arguments.length < 2) {
    throw new TypeError('2 arguments required, but only ' + arguments.length + ' present');
  }

  var dateLeftStartOfDay = startOfDay(dirtyDateLeft);
  var dateRightStartOfDay = startOfDay(dirtyDateRight);
  return dateLeftStartOfDay.getTime() === dateRightStartOfDay.getTime();
}

function isSameWeek(dirtyDateLeft, dirtyDateRight) {
  if (arguments.length < 2) {
    throw new TypeError('2 arguments required, but only ' + arguments.length + ' present');
  }

  var dateLeftStartOfWeek = startOfWeek(dirtyDateLeft);
  var dateRightStartOfWeek = startOfWeek(dirtyDateRight);
  return dateLeftStartOfWeek.getTime() === dateRightStartOfWeek.getTime();
}

function isSameMonth(dirtyDateLeft, dirtyDateRight) {
  if (arguments.length < 2) {
    throw new TypeError('2 arguments required, but only ' + arguments.length + ' present');
  }

  var dateLeft = toDate(dirtyDateLeft);
  var dateRight = toDate(dirtyDateRight);
  return dateLeft.getFullYear() === dateRight.getFullYear() && dateLeft.getMonth() === dateRight.getMonth();
}

function isSameYear(dirtyDateLeft, dirtyDateRight) {
  if (arguments.length < 2) {
    throw new TypeError('2 arguments required, but only ' + arguments.length + ' present');
  }

  var dateLeft = toDate(dirtyDateLeft);
  var dateRight = toDate(dirtyDateRight);
  return dateLeft.getFullYear() === dateRight.getFullYear();
}

function compareAsc(dirtyDateLeft, dirtyDateRight) {
  if (arguments.length < 2) {
    throw new TypeError('2 arguments required, but only ' + arguments.length + ' present');
  }

  var dateLeft = toDate(dirtyDateLeft);
  var dateRight = toDate(dirtyDateRight);
  var diff = dateLeft.getTime() - dateRight.getTime();

  if (diff < 0) {
    return -1;
  } else if (diff > 0) {
    return 1;
  } else {
    return diff;
  }
}

function getDayNumberOfMouth(dirtyDate) {
  var date = toDate(dirtyDate);
  var year = date.getFullYear();
  var monthIndex = date.getMonth();
  var lastDayOfMonth = new Date(0);
  lastDayOfMonth.setFullYear(year, monthIndex + 1, 0);
  lastDayOfMonth.setHours(0, 0, 0, 0);
  return lastDayOfMonth.getDate();
}

function getWeekNumberOfYear(dirtyDate) {
  var date = toDate(dirtyDate);
  date.setHours(0, 0, 0, 0);
  date.setDate(date.getDate() + 3 - (date.getDay() + 6) % 7);
  var week1 = new Date(date.getFullYear(), 0, 4);
  return 1 + Math.round(((date.getTime() - week1.getTime()) / 86400000 - 3 + (week1.getDay() + 6) % 7) / 7);
}

function prevMonth(dirtyDate) {
  return addMonths(dirtyDate, -1);
}

function nextMonth(dirtyDate) {
  return addMonths(dirtyDate, 1);
}

function prevYear(dirtyDate) {
  return addMonths(dirtyDate, -12);
}

function nextYear(dirtyDate) {
  return addMonths(dirtyDate, 12);
}

function getDaysOfMonth(dirtyDate) {
  var date = toDate(dirtyDate);
  var end = endOfWeek(endOfMonth(date));
  var current = startOfWeek(startOfMonth(date));
  current.setHours(date.getHours());
  current.setMinutes(date.getMinutes());
  current.setSeconds(date.getSeconds());
  var days = [];

  while (current.getTime() < end.getTime()) {
    days.push(current);
    current = addDays(current, 1);
  }

  return days;
}

var formatFlags = {
  D: function D(date) {
    return date.getDay();
  },
  DD: function DD(date) {
    return pad(date.getDay());
  },
  Do: function Do(date) {
    return getDaySuffix(date.getDate());
  },
  d: function d(date) {
    return date.getDate();
  },
  dd: function dd(date) {
    return pad(date.getDate());
  },
  M: function M(date) {
    return date.getMonth() + 1;
  },
  MM: function MM(date) {
    return pad(date.getMonth() + 1);
  },
  yy: function yy(date) {
    return String(date.getFullYear()).substr(2);
  },
  yyyy: function yyyy(date) {
    return date.getFullYear();
  },
  h: function h(date) {
    return date.getHours() % 12 || 12;
  },
  hh: function hh(date) {
    return pad(date.getHours() % 12 || 12);
  },
  H: function H(date) {
    return date.getHours();
  },
  HH: function HH(date) {
    return pad(date.getHours());
  },
  m: function m(date) {
    return date.getMinutes();
  },
  mm: function mm(date) {
    return pad(date.getMinutes());
  },
  s: function s(date) {
    return date.getSeconds();
  },
  ss: function ss(date) {
    return pad(date.getSeconds());
  },
  S: function S(date) {
    return Math.round(date.getMilliseconds() / 100);
  },
  SS: function SS(date) {
    return pad(Math.round(date.getMilliseconds() / 10), 2);
  },
  SSS: function SSS(date) {
    return pad(date.getMilliseconds(), 3);
  },
  a: function a(date) {
    return date.getHours() < 12 ? amPm[0] : amPm[1];
  },
  A: function A(date) {
    return date.getHours() < 12 ? amPm[0].toUpperCase() : amPm[1].toUpperCase();
  },
  WW: function WW(date) {
    return pad(getWeekNumberOfYear(date));
  },
  Wo: function Wo(date) {
    return getDaySuffix(getWeekNumberOfYear(date));
  },
  ZZ: function ZZ(date) {
    var o = date.getTimezoneOffset();
    return (o > 0 ? '-' : '+') + pad(Math.floor(Math.abs(o) / 60) * 100 + Math.abs(o) % 60, 4);
  }
};
var parseFlags = {
  d: [twoDigits, function (d, v) {
    d.day = v;
  }],
  M: [twoDigits, function (d, v) {
    d.month = v - 1;
  }],
  h: [twoDigits, function (d, v) {
    d.hour = v;
  }],
  m: [twoDigits, function (d, v) {
    d.minute = v;
  }],
  s: [twoDigits, function (d, v) {
    d.second = v;
  }],
  yy: [twoDigits, function (d, v) {
    var da = new Date();
    var cent = +('' + da.getFullYear()).substr(0, 2);
    d.year = '' + (v > 68 ? cent - 1 : cent) + v;
  }],
  yyyy: [fourDigits, function (d, v) {
    d.year = v;
  }],
  S: [/\d/, function (d, v) {
    d.millisecond = v * 100;
  }],
  SS: [/\d{2}/, function (d, v) {
    d.millisecond = v * 10;
  }],
  SSS: [threeDigits, function (d, v) {
    d.millisecond = v;
  }],
  D: [twoDigits],
  ddd: [word],
  a: [word, function (d, v) {
    var val = v.toLowerCase();

    if (val === amPm[0]) {
      d.isPm = false;
    } else if (val === amPm[1]) {
      d.isPm = true;
    }
  }],
  ZZ: [/[\+\-]\d\d:?\d\d/, function (d, v) {
    var parts = (v + '').match(/([\+\-]|\d\d)/gi);

    if (parts && parts[1]) {
      var minutes = +(parts[1] * 60) + parseInt(parts[2], 10);
      d.timezoneOffset = parts[0] === '+' ? minutes : -minutes;
    }
  }]
};
parseFlags.DD = parseFlags.D;
parseFlags.Do = parseFlags.dd = parseFlags.d;
parseFlags.mm = parseFlags.m;
parseFlags.hh = parseFlags.H = parseFlags.HH = parseFlags.h;
parseFlags.MM = parseFlags.M;
parseFlags.ss = parseFlags.s;
parseFlags.A = parseFlags.a;

function formatDate(dirtyDate, fmt) {
  if (arguments.length < 2) {
    throw new TypeError('2 argument required, but only ' + arguments.length + ' present');
  }

  var date = toDate(dirtyDate);
  var literals = [];
  fmt = fmt.replace(literal, function ($0, $1) {
    literals.push($1);
    return '??';
  });
  fmt = fmt.replace(token, function ($0) {
    return $0 in formatFlags ? formatFlags[$0](date) : $0.slice(1, $0.length - 1);
  });
  return fmt.replace(/\?\?/g, function () {
    return literals.shift();
  });
}

function parseDate(dirtyDateString, dirtyFormatString) {
  if (arguments.length < 2) {
    throw new TypeError('2 argument required, but only ' + arguments.length + ' present');
  }

  dirtyDateString = String(dirtyDateString);
  dirtyFormatString = String(dirtyFormatString);
  var dateInfo = {};
  var isValid = true;
  dirtyFormatString.replace(token, function ($0) {
    if (parseFlags[$0]) {
      var info = parseFlags[$0];
      var index = dirtyDateString.search(info[0]); // tslint:disable-next-line

      if (!~index) {
        isValid = false;
      } else {
        dirtyDateString.replace(info[0], function (result) {
          if (typeof info[1] === 'function') {
            info[1](dateInfo, result);
          }

          dirtyDateString = dirtyDateString.substr(index + result.length);
          return result;
        });
      }
    }

    return parseFlags[$0] ? '' : $0.slice(1, $0.length - 1);
  });

  if (!isValid) {
    return toDate(dirtyFormatString);
  }

  var today = new Date();

  if (dateInfo.isPm === true && dateInfo.hour != null && +dateInfo.hour !== 12) {
    dateInfo.hour = +dateInfo.hour + 12;
  } else if (dateInfo.isPm === false && +dateInfo.hour === 12) {
    dateInfo.hour = 0;
  }

  var date;

  if (dateInfo.timezoneOffset != null) {
    dateInfo.minute = +(dateInfo.minute || 0) - +dateInfo.timezoneOffset;
    date = new Date(Date.UTC(dateInfo.year || today.getFullYear(), dateInfo.month || 0, dateInfo.day || 1, dateInfo.hour || 0, dateInfo.minute || 0, dateInfo.second || 0, dateInfo.millisecond || 0));
  } else {
    date = new Date(dateInfo.year || today.getFullYear(), dateInfo.month || 0, dateInfo.day || 1, dateInfo.hour || 0, dateInfo.minute || 0, dateInfo.second || 0, dateInfo.millisecond || 0);
  }

  return date;
}